@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-#
 # Copyright 2014 - Mirantis, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
@@ -16,10 +14,6 @@
 
 import abc
 
-from mistral.openstack.common import log as logging
-
-LOG = logging.getLogger(__name__)
-
 
 class Action(object):
     """Action.
@@ -47,12 +41,21 @@ class Action(object):
     def run(self):
         """Run action logic.
 
-        :return: result of the action. Note that for asynchronous actions
+        :return: Result of the action. Note that for asynchronous actions
         it should always be None, however, if even it's not None it will be
         ignored by a caller.
 
-        In case if action failed this method must throw a ActionException
-        to indicate that.
+        Result can be of two types:
+        1) Any serializable value meaningful from a user perspective (such
+        as string, number or dict).
+        2) Instance of {mistral.workflow.utils.Result} which has field "data"
+        for success result and field "error" for keeping so called "error
+        result" like HTTP error code and similar. Using the second type
+        allows to communicate a result even in case of error and hence to have
+        conditions in "on-error" clause of direct workflows. Depending on
+        particular action semantics one or another option may be preferable.
+        In case if action failed and there's no need to communicate any error
+        result this method should throw a ActionException.
         """
         pass
 
