@@ -13,12 +13,13 @@
 #    limitations under the License.
 
 import json
+import os
 
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_log import log as logging
 import pkg_resources as pkg
 
 from mistral.actions import action_generator
-from mistral.openstack.common import log as logging
 from mistral.utils import inspect_utils as i_u
 from mistral import version
 
@@ -29,7 +30,6 @@ os_actions_mapping_path = cfg.StrOpt('openstack_actions_mapping_path',
 CONF = cfg.CONF
 CONF.register_opt(os_actions_mapping_path)
 LOG = logging.getLogger(__name__)
-MAPPING_PATH = CONF.openstack_actions_mapping_path
 
 
 def get_mapping():
@@ -39,10 +39,19 @@ def get_mapping():
                 delete_comment(value)
         if '_comment' in map_part:
             del map_part['_comment']
+    package = version.version_info.package
 
-    mapping = json.loads(open(pkg.resource_filename(
-                         version.version_info.package,
-                         MAPPING_PATH)).read())
+    if os.path.isabs(CONF.openstack_actions_mapping_path):
+        mapping_file_path = CONF.openstack_actions_mapping_path
+    else:
+        path = CONF.openstack_actions_mapping_path
+        mapping_file_path = pkg.resource_filename(package, path)
+
+    LOG.info("Processing OpenStack action mapping from file: %s",
+             mapping_file_path)
+
+    with open(mapping_file_path) as fh:
+        mapping = json.load(fh)
 
     for k, v in mapping.items():
         if isinstance(v, dict):
@@ -85,7 +94,8 @@ class OpenStackActionGenerator(action_generator.ActionGenerator):
             try:
                 client_method = clazz.get_fake_client_method()
             except Exception as e:
-                LOG.debug("Failed to get fake client method: %s" % e)
+                LOG.warning("Failed to create action: %s.%s %s" %
+                            (cls.action_namespace, action_name, e))
                 client_method = None
 
             if client_method:
