@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
+# Copyright 2016 - Brocade Communications Systems, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -17,6 +17,7 @@ import sys
 
 import eventlet
 
+
 eventlet.monkey_patch(
     os=True,
     select=True,
@@ -24,8 +25,10 @@ eventlet.monkey_patch(
     thread=False if '--use-debugger' in sys.argv else True,
     time=True)
 
+
 import os
 
+
 # If ../mistral/__init__.py exists, add ../ to Python search path, so that
 # it will override what happens to be installed in /usr/(local/)lib/python...
 POSSIBLE_TOPDIR = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
@@ -34,102 +37,73 @@ POSSIBLE_TOPDIR = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
 if os.path.exists(os.path.join(POSSIBLE_TOPDIR, 'mistral', '__init__.py')):
     sys.path.insert(0, POSSIBLE_TOPDIR)
 
-from oslo.config import cfg
-from oslo import messaging
-from wsgiref import simple_server
+from oslo_config import cfg
+from oslo_log import log as logging
+from oslo_service import service
 
-from mistral.api import app
+from mistral.api import service as api_service
 from mistral import config
-from mistral import context as ctx
-from mistral.db.v2 import api as db_api
-from mistral import engine
-from mistral.engine import executor
-from mistral.engine1 import default_engine as def_eng
-from mistral.engine1 import default_executor as def_executor
-from mistral.engine1 import rpc
-from mistral.openstack.common import log as logging
-from mistral.services import scheduler
+from mistral.engine import engine_server
+from mistral.engine import executor_server
+from mistral.engine.rpc_backend import rpc
+from mistral.event_engine import event_engine_server
+from mistral import version
 
 
-LOG = logging.getLogger(__name__)
+CONF = cfg.CONF
 
 
-def launch_executor(transport):
-    target = messaging.Target(
-        topic=cfg.CONF.executor.topic,
-        server=cfg.CONF.executor.host
-    )
+def launch_executor():
+    try:
+        launcher = service.ServiceLauncher(CONF)
 
-    # Since engine and executor are tightly coupled, use the engine
-    # configuration to decide which executor to get.
-    executor_v1 = executor.get_executor(cfg.CONF.engine.engine, transport)
+        launcher.launch_service(executor_server.get_oslo_service())
 
-    executor_v2 = def_executor.DefaultExecutor(rpc.get_engine_client())
-
-    endpoints = [executor_v1, rpc.ExecutorServer(executor_v2)]
-
-    server = messaging.get_rpc_server(
-        transport,
-        target,
-        endpoints,
-        executor='eventlet',
-        serializer=ctx.RpcContextSerializer(ctx.JsonPayloadSerializer())
-    )
-
-    server.start()
-    server.wait()
+        launcher.wait()
+    except RuntimeError as e:
+        sys.stderr.write("ERROR: %s\n" % e)
+        sys.exit(1)
 
 
-def launch_engine(transport):
-    target = messaging.Target(
-        topic=cfg.CONF.engine.topic,
-        server=cfg.CONF.engine.host
-    )
+def launch_engine():
+    try:
+        launcher = service.ServiceLauncher(CONF)
 
-    engine_v1 = engine.get_engine(cfg.CONF.engine.engine, transport)
-    engine_v2 = def_eng.DefaultEngine(rpc.get_engine_client())
+        launcher.launch_service(engine_server.get_oslo_service())
 
-    endpoints = [engine_v1, rpc.EngineServer(engine_v2)]
-
-    # Setup scheduler in engine.
-    db_api.setup_db()
-    scheduler.setup()
-
-    server = messaging.get_rpc_server(
-        transport,
-        target,
-        endpoints,
-        executor='eventlet',
-        serializer=ctx.RpcContextSerializer(ctx.JsonPayloadSerializer())
-    )
-
-    server.start()
-    server.wait()
+        launcher.wait()
+    except RuntimeError as e:
+        sys.stderr.write("ERROR: %s\n" % e)
+        sys.exit(1)
 
 
-def launch_api(transport):
-    host = cfg.CONF.api.host
-    port = cfg.CONF.api.port
+def launch_event_engine():
+    try:
+        launcher = service.ServiceLauncher(CONF)
 
-    server = simple_server.make_server(
-        host,
-        port,
-        app.setup_app(transport=transport)
-    )
+        launcher.launch_service(event_engine_server.get_oslo_service())
 
-    LOG.info("Mistral API is serving on http://%s:%s (PID=%s)" %
-             (host, port, os.getpid()))
-
-    server.serve_forever()
+        launcher.wait()
+    except RuntimeError as e:
+        sys.stderr.write("ERROR: %s\n" % e)
+        sys.exit(1)
 
 
-def launch_any(transport, options):
+def launch_api():
+    launcher = service.ProcessLauncher(cfg.CONF)
+
+    server = api_service.WSGIService('mistral_api')
+
+    launcher.launch_service(server, workers=server.workers)
+
+    launcher.wait()
+
+
+def launch_any(options):
     # Launch the servers on different threads.
-    threads = [eventlet.spawn(LAUNCH_OPTIONS[option], transport)
+    threads = [eventlet.spawn(LAUNCH_OPTIONS[option])
                for option in options]
 
-    print('Server started.')
-
     [thread.wait() for thread in threads]
 
 
@@ -138,7 +112,8 @@ def launch_any(transport, options):
 LAUNCH_OPTIONS = {
     'api': launch_api,
     'engine': launch_engine,
-    'executor': launch_executor
+    'executor': launch_executor,
+    'event-engine': launch_event_engine
 }
 
 
@@ -150,26 +125,50 @@ MISTRAL_TITLE = """
 ||      || ||   ||   ||   ||  ||  //      \\\  ||
 ||      || || _//    ||   ||  || //        \\\ |||||
 
-Mistral Workflow Service, version 0.1, 2014
-"""
+Mistral Workflow Service, version %s
+""" % version.version_string()
 
 
 def print_server_info():
     print(MISTRAL_TITLE)
 
-    comp_str = "[%s]" % ','.join(LAUNCH_OPTIONS) \
-        if cfg.CONF.server == ['all'] else cfg.CONF.server
+    comp_str = ("[%s]" % ','.join(LAUNCH_OPTIONS)
+                if cfg.CONF.server == ['all'] else cfg.CONF.server)
 
     print('Launching server components %s...' % comp_str)
 
 
+def get_properly_ordered_parameters():
+    """Orders launch parameters in the right order.
+
+    In oslo it's important the order of the launch parameters.
+    if --config-file came after the command line parameters the command
+    line parameters are ignored.
+    So to make user command line parameters are never ignored this method
+    moves --config-file to be always first.
+    """
+    args = sys.argv[1:]
+
+    for arg in sys.argv[1:]:
+        if arg == '--config-file' or arg.startswith('--config-file='):
+            if "=" in arg:
+                conf_file_value = arg.split("=", 1)[1]
+            else:
+                conf_file_value = args[args.index(arg) + 1]
+                args.remove(conf_file_value)
+            args.remove(arg)
+            args.insert(0, "--config-file")
+            args.insert(1, conf_file_value)
+
+    return args
+
+
 def main():
     try:
-        config.parse_args()
-
+        config.parse_args(get_properly_ordered_parameters())
         print_server_info()
 
-        logging.setup('Mistral')
+        logging.setup(CONF, 'Mistral')
 
         # Please refer to the oslo.messaging documentation for transport
         # configuration. The default transport for oslo.messaging is
@@ -186,19 +185,19 @@ def main():
         # servers are launched on the same process. Otherwise, messages do not
         # get delivered if the Mistral servers are launched on different
         # processes because the "fake" transport is using an in process queue.
-        transport = rpc.get_transport()
+        rpc.get_transport()
 
         if cfg.CONF.server == ['all']:
             # Launch all servers.
-            launch_any(transport, LAUNCH_OPTIONS.keys())
+            launch_any(LAUNCH_OPTIONS.keys())
         else:
             # Validate launch option.
             if set(cfg.CONF.server) - set(LAUNCH_OPTIONS.keys()):
                 raise Exception('Valid options are all or any combination of '
-                                'api, engine, and executor.')
+                                ', '.join(LAUNCH_OPTIONS.keys()))
 
             # Launch distinct set of server(s).
-            launch_any(transport, set(cfg.CONF.server))
+            launch_any(set(cfg.CONF.server))
 
     except RuntimeError as excp:
         sys.stderr.write("ERROR: %s\n" % excp)
@@ -206,4 +205,4 @@ def main():
 
 
 if __name__ == '__main__':
-    main()
+    sys.exit(main())
