@@ -16,14 +16,13 @@
 import contextlib
 
 from oslo_db import api as db_api
-from oslo_log import log as logging
+
 
 _BACKEND_MAPPING = {
     'sqlalchemy': 'mistral.db.v2.sqlalchemy.api',
 }
 
 IMPL = db_api.DBAPI('sqlalchemy', backend_mapping=_BACKEND_MAPPING)
-LOG = logging.getLogger(__name__)
 
 
 def setup_db():
@@ -59,6 +58,10 @@ def transaction():
         yield
 
 
+def refresh(model):
+    IMPL.refresh(model)
+
+
 # Locking.
 
 
@@ -77,8 +80,16 @@ def load_workbook(name):
     return IMPL.load_workbook(name)
 
 
-def get_workbooks():
-    return IMPL.get_workbooks()
+def get_workbooks(limit=None, marker=None, sort_keys=None,
+                  sort_dirs=None, fields=None, **kwargs):
+    return IMPL.get_workbooks(
+        limit=limit,
+        marker=marker,
+        sort_keys=sort_keys,
+        sort_dirs=sort_dirs,
+        fields=fields,
+        **kwargs
+    )
 
 
 def create_workbook(values):
@@ -103,8 +114,8 @@ def delete_workbooks(**kwargs):
 
 # Workflow definitions.
 
-def get_workflow_definition(name):
-    return IMPL.get_workflow_definition(name)
+def get_workflow_definition(identifier):
+    return IMPL.get_workflow_definition(identifier)
 
 
 def get_workflow_definition_by_id(id):
@@ -132,16 +143,16 @@ def create_workflow_definition(values):
     return IMPL.create_workflow_definition(values)
 
 
-def update_workflow_definition(name, values):
-    return IMPL.update_workflow_definition(name, values)
+def update_workflow_definition(identifier, values):
+    return IMPL.update_workflow_definition(identifier, values)
 
 
 def create_or_update_workflow_definition(name, values):
     return IMPL.create_or_update_workflow_definition(name, values)
 
 
-def delete_workflow_definition(name):
-    IMPL.delete_workflow_definition(name)
+def delete_workflow_definition(identifier):
+    IMPL.delete_workflow_definition(identifier)
 
 
 def delete_workflow_definitions(**kwargs):
@@ -178,8 +189,8 @@ def create_action_definition(values):
     return IMPL.create_action_definition(values)
 
 
-def update_action_definition(name, values):
-    return IMPL.update_action_definition(name, values)
+def update_action_definition(identifier, values):
+    return IMPL.update_action_definition(identifier, values)
 
 
 def create_or_update_action_definition(name, values):
@@ -194,45 +205,6 @@ def delete_action_definitions(**kwargs):
     return IMPL.delete_action_definitions(**kwargs)
 
 
-# Common executions.
-
-def get_execution(id):
-    return IMPL.get_execution(id)
-
-
-def load_execution(name):
-    """Unlike get_execution this method is allowed to return None."""
-    return IMPL.load_execution(name)
-
-
-def get_executions(**kwargs):
-    return IMPL.get_executions(**kwargs)
-
-
-def ensure_execution_exists(id):
-    return IMPL.ensure_execution_exists(id)
-
-
-def create_execution(values):
-    return IMPL.create_execution(values)
-
-
-def update_execution(id, values):
-    return IMPL.update_execution(id, values)
-
-
-def create_or_update_execution(id, values):
-    return IMPL.create_or_update_execution(id, values)
-
-
-def delete_execution(id):
-    return IMPL.delete_execution(id)
-
-
-def delete_executions(**kwargs):
-    IMPL.delete_executions(**kwargs)
-
-
 # Action executions.
 
 def get_action_execution(id):
@@ -324,13 +296,32 @@ def get_task_execution(id):
     return IMPL.get_task_execution(id)
 
 
-def load_task_execution(name):
+def load_task_execution(id):
     """Unlike get_task_execution this method is allowed to return None."""
-    return IMPL.load_task_execution(name)
+    return IMPL.load_task_execution(id)
 
 
-def get_task_executions(**kwargs):
-    return IMPL.get_task_executions(**kwargs)
+def get_task_executions(limit=None, marker=None, sort_keys=['created_at'],
+                        sort_dirs=None, **kwargs):
+    return IMPL.get_task_executions(
+        limit=limit,
+        marker=marker,
+        sort_keys=sort_keys,
+        sort_dirs=sort_dirs,
+        **kwargs
+    )
+
+
+def get_completed_task_executions(**kwargs):
+    return IMPL.get_completed_task_executions(**kwargs)
+
+
+def get_incomplete_task_executions(**kwargs):
+    return IMPL.get_incomplete_task_executions(**kwargs)
+
+
+def get_incomplete_task_executions_count(**kwargs):
+    return IMPL.get_incomplete_task_executions_count(**kwargs)
 
 
 def create_task_execution(values):
@@ -375,6 +366,14 @@ def get_delayed_call(id):
     return IMPL.get_delayed_call(id)
 
 
+def get_delayed_calls(**kwargs):
+    return IMPL.get_delayed_calls(**kwargs)
+
+
+def delete_delayed_calls(**kwargs):
+    return IMPL.delete_delayed_calls(**kwargs)
+
+
 # Cron triggers.
 
 def get_cron_trigger(name):
@@ -429,8 +428,16 @@ def load_environment(name):
     return IMPL.load_environment(name)
 
 
-def get_environments():
-    return IMPL.get_environments()
+def get_environments(limit=None, marker=None, sort_keys=['name'],
+                     sort_dirs=None, **kwargs):
+
+    return IMPL.get_environments(
+        limit=limit,
+        marker=marker,
+        sort_keys=sort_keys,
+        sort_dirs=sort_dirs,
+        **kwargs
+    )
 
 
 def create_environment(values):
@@ -451,3 +458,94 @@ def delete_environment(name):
 
 def delete_environments(**kwargs):
     IMPL.delete_environments(**kwargs)
+
+
+# Resource members.
+
+
+def create_resource_member(values):
+    return IMPL.create_resource_member(values)
+
+
+def get_resource_member(resource_id, res_type, member_id):
+    return IMPL.get_resource_member(resource_id, res_type, member_id)
+
+
+def get_resource_members(resource_id, res_type):
+    return IMPL.get_resource_members(resource_id, res_type)
+
+
+def update_resource_member(resource_id, res_type, member_id, values):
+    return IMPL.update_resource_member(
+        resource_id,
+        res_type,
+        member_id,
+        values
+    )
+
+
+def delete_resource_member(resource_id, res_type, member_id):
+    IMPL.delete_resource_member(resource_id, res_type, member_id)
+
+
+def delete_resource_members(**kwargs):
+    IMPL.delete_resource_members(**kwargs)
+
+
+# Event triggers.
+
+def get_event_trigger(id, insecure=False):
+    return IMPL.get_event_trigger(id, insecure)
+
+
+def get_event_triggers(insecure=False, limit=None, marker=None, sort_keys=None,
+                       sort_dirs=None, fields=None, **kwargs):
+    return IMPL.get_event_triggers(
+        insecure=False,
+        limit=limit,
+        marker=marker,
+        sort_keys=sort_keys,
+        sort_dirs=sort_dirs,
+        fields=fields,
+        **kwargs
+    )
+
+
+def create_event_trigger(values):
+    return IMPL.create_event_trigger(values)
+
+
+def update_event_trigger(id, values):
+    return IMPL.update_event_trigger(id, values)
+
+
+def delete_event_trigger(id):
+    return IMPL.delete_event_trigger(id)
+
+
+def delete_event_triggers(**kwargs):
+    return IMPL.delete_event_triggers(**kwargs)
+
+
+def ensure_event_trigger_exists(id):
+    return IMPL.ensure_event_trigger_exists(id)
+
+
+# Locks.
+
+def create_named_lock(name):
+    return IMPL.create_named_lock(name)
+
+
+def get_named_locks(limit=None, marker=None):
+    return IMPL.get_named_locks(limit=limit, marker=marker)
+
+
+def delete_named_lock(lock_id):
+    return IMPL.delete_named_lock(lock_id)
+
+
+@contextlib.contextmanager
+def named_lock(name):
+    with IMPL.named_lock(name):
+        yield
