@@ -14,37 +14,31 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
-from oslo_log import log as logging
 from osprofiler import profiler
 
-from mistral import coordination
+from mistral.db import utils as db_utils
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models as db_models
 from mistral.engine import action_handler
+from mistral.engine import action_queue
 from mistral.engine import base
 from mistral.engine import workflow_handler as wf_handler
 from mistral import exceptions
 from mistral import utils as u
 from mistral.workflow import states
 
-LOG = logging.getLogger(__name__)
-
 
 # Submodules of mistral.engine will throw NoSuchOptError if configuration
 # options required at top level of this  __init__.py are not imported before
 # the submodules are referenced.
 
 
-class DefaultEngine(base.Engine, coordination.Service):
-    def __init__(self, engine_client):
-        self._engine_client = engine_client
-
-        coordination.Service.__init__(self, 'engine_group')
-
-    @u.log_exec(LOG)
+class DefaultEngine(base.Engine):
+    @action_queue.process
     @profiler.trace('engine-start-workflow')
     def start_workflow(self, wf_identifier, wf_input, description='',
                        **params):
+
         with db_api.transaction():
             wf_ex = wf_handler.start_workflow(
                 wf_identifier,
@@ -55,7 +49,7 @@ class DefaultEngine(base.Engine, coordination.Service):
 
             return wf_ex.get_clone()
 
-    @u.log_exec(LOG)
+    @action_queue.process
     def start_action(self, action_name, action_input,
                      description=None, **params):
         with db_api.transaction():
@@ -106,9 +100,11 @@ class DefaultEngine(base.Engine, coordination.Service):
 
             return db_api.create_action_execution(values)
 
-    @u.log_exec(LOG)
-    @profiler.trace('engine-on-action-complete')
-    def on_action_complete(self, action_ex_id, result, wf_action=False):
+    @db_utils.retry_on_deadlock
+    @action_queue.process
+    @profiler.trace('engine-on-action-complete', hide_args=True)
+    def on_action_complete(self, action_ex_id, result, wf_action=False,
+                           async_=False):
         with db_api.transaction():
             if wf_action:
                 action_ex = db_api.get_workflow_execution(action_ex_id)
@@ -119,7 +115,6 @@ class DefaultEngine(base.Engine, coordination.Service):
 
             return action_ex.get_clone()
 
-    @u.log_exec(LOG)
     def pause_workflow(self, wf_ex_id):
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex_id)
@@ -128,7 +123,7 @@ class DefaultEngine(base.Engine, coordination.Service):
 
             return wf_ex.get_clone()
 
-    @u.log_exec(LOG)
+    @action_queue.process
     def rerun_workflow(self, task_ex_id, reset=True, env=None):
         with db_api.transaction():
             task_ex = db_api.get_task_execution(task_ex_id)
@@ -139,7 +134,7 @@ class DefaultEngine(base.Engine, coordination.Service):
 
             return wf_ex.get_clone()
 
-    @u.log_exec(LOG)
+    @action_queue.process
     def resume_workflow(self, wf_ex_id, env=None):
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex_id)
@@ -148,7 +143,6 @@ class DefaultEngine(base.Engine, coordination.Service):
 
             return wf_ex.get_clone()
 
-    @u.log_exec(LOG)
     def stop_workflow(self, wf_ex_id, state, message=None):
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex_id)
@@ -157,7 +151,6 @@ class DefaultEngine(base.Engine, coordination.Service):
 
             return wf_ex.get_clone()
 
-    @u.log_exec(LOG)
     def rollback_workflow(self, wf_ex_id):
         # TODO(rakhmerov): Implement.
         raise NotImplementedError
