@@ -15,42 +15,48 @@
 
 import inspect
 
+from oslo_log import log as logging
 from stevedore import extension
 
 from mistral.actions import action_factory
 from mistral.actions import generator_factory
 from mistral.db.v2 import api as db_api
 from mistral import exceptions as exc
-from mistral import expressions as expr
-from mistral.openstack.common import log as logging
 from mistral.services import actions
 from mistral import utils
 from mistral.utils import inspect_utils as i_utils
 
 
+# TODO(rakhmerov): Make methods more consistent and granular.
+
 LOG = logging.getLogger(__name__)
 
-ACTIONS_PATH = '../resources/actions'
+ACTIONS_PATH = 'resources/actions'
 _ACTION_CTX_PARAM = 'action_context'
 
 
+# TODO(rakhmerov): It's confusing because we have std.xxx actions and actions
+# TODO(rakhmerov): under '../resources/actions' that we also call standard.
 def register_standard_actions():
     action_paths = utils.get_file_list(ACTIONS_PATH)
 
     for action_path in action_paths:
         action_definition = open(action_path).read()
-        actions.update_actions(action_definition, scope='public')
+        actions.create_or_update_actions(
+            action_definition,
+            scope='public'
+        )
 
 
 def get_registered_actions(**kwargs):
-    return db_api.get_actions(**kwargs)
+    return db_api.get_action_definitions(**kwargs)
 
 
-def _register_action_in_db(name, action_class, attributes,
-                           description=None, input_str=None):
+def register_action_class(name, action_class_str, attributes,
+                          description=None, input_str=None):
     values = {
         'name': name,
-        'action_class': action_class,
+        'action_class': action_class_str,
         'attributes': attributes,
         'description': description,
         'input': input_str,
@@ -61,19 +67,20 @@ def _register_action_in_db(name, action_class, attributes,
     try:
         LOG.debug("Registering action in DB: %s" % name)
 
-        db_api.create_action(values)
-    except exc.DBDuplicateEntry:
+        db_api.create_action_definition(values)
+    except exc.DBDuplicateEntryError:
         LOG.debug("Action %s already exists in DB." % name)
 
 
 def _clear_system_action_db():
-    db_api.delete_actions(is_system=True)
+    db_api.delete_action_definitions(is_system=True)
 
 
 def sync_db():
-    _clear_system_action_db()
-    register_action_classes()
-    register_standard_actions()
+    with db_api.transaction():
+        _clear_system_action_db()
+        register_action_classes()
+        register_standard_actions()
 
 
 def _register_dynamic_action_classes():
@@ -88,7 +95,7 @@ def _register_dynamic_action_classes():
         for action in actions:
             attrs = i_utils.get_public_fields(action['class'])
 
-            _register_action_in_db(
+            register_action_class(
                 action['name'],
                 action_class_str,
                 attrs,
@@ -102,25 +109,27 @@ def register_action_classes():
         namespace='mistral.actions',
         invoke_on_load=False
     )
+    for name in mgr.names():
+        action_class_str = mgr[name].entry_point_target.replace(':', '.')
+        action_class = mgr[name].plugin
+        description = i_utils.get_docstring(action_class)
+        input_str = i_utils.get_arg_list_as_str(action_class.__init__)
 
-    with db_api.transaction():
-        for name in mgr.names():
-            action_class_str = mgr[name].entry_point_target.replace(':', '.')
-            action_class = mgr[name].plugin
-            description = i_utils.get_docstring(action_class)
-            input_str = i_utils.get_arg_list_as_str(action_class.__init__)
+        attrs = i_utils.get_public_fields(mgr[name].plugin)
 
-            attrs = i_utils.get_public_fields(mgr[name].plugin)
+        register_action_class(
+            name,
+            action_class_str,
+            attrs,
+            description=description,
+            input_str=input_str
+        )
 
-            _register_action_in_db(name, action_class_str, attrs,
-                                   description=description,
-                                   input_str=input_str)
-
-        _register_dynamic_action_classes()
+    _register_dynamic_action_classes()
 
 
 def get_action_db(action_name):
-    return db_api.load_action(action_name)
+    return db_api.load_action_definition(action_name)
 
 
 def get_action_class(action_full_name):
@@ -132,62 +141,63 @@ def get_action_class(action_full_name):
     action_db = get_action_db(action_full_name)
 
     if action_db:
-        return action_factory.construct_action_class(action_db.action_class,
-                                                     action_db.attributes)
+        return action_factory.construct_action_class(
+            action_db.action_class,
+            action_db.attributes
+        )
 
 
-def get_action_context(task_db):
+def get_action_context(task_ex, action_ex_id, save=True):
+    if task_ex:
+        return {
+            _ACTION_CTX_PARAM: {
+                'workflow_name': task_ex.workflow_name,
+                'workflow_execution_id': task_ex.workflow_execution_id,
+                'task_id': task_ex.id,
+                'task_name': task_ex.name,
+                'task_tags': task_ex.tags,
+                'action_execution_id': action_ex_id,
+                'callback_url': '/v2/action_executions/%s' % action_ex_id
+            }
+        }
+    elif save:
+        return {
+            _ACTION_CTX_PARAM: {
+                'workflow_name': None,
+                'workflow_execution_id': None,
+                'task_id': None,
+                'task_name': None,
+                'task_tags': None,
+                'action_execution_id': action_ex_id,
+                'callback_url': '/v2/action_executions/%s' % action_ex_id
+            }
+        }
+
     return {
         _ACTION_CTX_PARAM: {
-            'workflow_name': task_db.execution.wf_name,
-            'execution_id': task_db.execution_id,
-            'task_id': task_db.id,
-            'task_name': task_db.name,
-            'task_tags': task_db.tags
+            'workflow_name': None,
+            'workflow_execution_id': None,
+            'task_id': None,
+            'task_name': None,
+            'task_tags': None,
+            'action_execution_id': None,
+            'callback_url': None
         }
     }
 
 
-def has_action_context(action, attributes):
+def get_empty_action_context():
+    return {
+        _ACTION_CTX_PARAM: {}
+    }
+
+
+def _has_argument(action, attributes, argument_name):
     action_cls = action_factory.construct_action_class(action, attributes)
     arg_spec = inspect.getargspec(action_cls.__init__)
-    return _ACTION_CTX_PARAM in arg_spec.args
+
+    return argument_name in arg_spec.args
 
 
-def resolve_adhoc_action_name(workbook, action_name):
-    action_spec = workbook.get_action(action_name)
-
-    if not action_spec:
-        msg = 'Ad-hoc action class is not registered ' \
-              '[workbook=%s, action=%s, action_spec=%s]' % \
-              (workbook, action_name, action_spec)
-        raise exc.ActionException(msg)
-
-    base_cls = get_action_class(action_spec.clazz)
-
-    if not base_cls:
-        msg = 'Ad-hoc action base class is not registered ' \
-              '[workbook=%s, action=%s, base_class=%s]' % \
-              (workbook, action_name, base_cls)
-        raise exc.ActionException(msg)
-
-    return action_spec.clazz
-
-
-def convert_adhoc_action_params(workbook, action_name, params):
-    base_params = workbook.get_action(action_name).base_parameters
-
-    if not base_params:
-        return {}
-
-    return expr.evaluate_recursively(base_params, params)
-
-
-def convert_adhoc_action_result(workbook, action_name, result):
-    transformer = workbook.get_action(action_name).output
-
-    if not transformer:
-        return result
-
-    # Use base action result as a context for evaluating expressions.
-    return expr.evaluate_recursively(transformer, result)
+def has_action_context(action, attributes):
+    return _has_argument(action, attributes, _ACTION_CTX_PARAM)
