@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-#
 # Copyright 2014 - Mirantis, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
@@ -15,7 +13,7 @@
 #    limitations under the License.
 
 from mistral.actions import std_actions as std
-from mistral.tests import base
+from mistral.tests.unit import base
 
 
 class EchoActionTest(base.BaseTest):
@@ -23,4 +21,4 @@ class EchoActionTest(base.BaseTest):
         expected = "my output"
         action = std.EchoAction(expected)
 
-        self.assertEqual(action.run(), expected)
+        self.assertEqual(expected, action.run())
