@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-#
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
 #    You may obtain a copy of the License at
@@ -15,15 +13,11 @@
 import datetime
 import uuid
 
-from keystonemiddleware import auth_token
-import mock
-from oslo.config import cfg
+from oslo_config import cfg
+from oslo_utils import timeutils
 import pecan
 import pecan.testing
 
-from mistral.db.v1 import api as db_api
-from mistral.db.v1.sqlalchemy import models
-from mistral.openstack.common import timeutils
 from mistral.tests.unit.api import base
 
 
@@ -42,8 +36,9 @@ PKI_TOKEN_VERIFIED = {
         'methods': ['password'],
         'roles': [{'id': uuid.uuid4().hex,
                    'name': 'admin'}],
-        'expires_at': timeutils.isotime(datetime.datetime.utcnow() +
-                                        datetime.timedelta(seconds=60)),
+        'expires_at': datetime.datetime.isoformat(
+            datetime.datetime.utcnow() + datetime.timedelta(seconds=60)
+        ),
         'project': {
             'domain': {'id': 'default', 'name': 'Default'},
             'id': uuid.uuid4().hex,
@@ -56,55 +51,29 @@ PKI_TOKEN_VERIFIED = {
             'id': uuid.uuid4().hex,
             'name': 'admin'
         },
-        'issued_at': timeutils.isotime()
+        'issued_at': datetime.datetime.isoformat(timeutils.utcnow())
     }
 }
 
 
-def get_mock_workbook(values):
-    wb = models.Workbook()
-    wb.update(values)
-    return wb
+class TestKeystoneMiddleware(base.APITest):
+    """Test keystone middleware AuthProtocol.
 
-
-class TestKeystoneMiddleware(base.FunctionalTest):
-    """Test that the keystone middleware AuthProtocol is executed
+    It checks that keystone middleware AuthProtocol is executed
     when enabled.
     """
 
     def setUp(self):
         super(TestKeystoneMiddleware, self).setUp()
+
         cfg.CONF.set_default('auth_enable', True, group='pecan')
+
         self.app = pecan.testing.load_test_app({
             'app': {
                 'root': cfg.CONF.pecan.root,
                 'modules': cfg.CONF.pecan.modules,
                 'debug': cfg.CONF.pecan.debug,
-                'auth_enable': cfg.CONF.pecan.auth_enable
+                'auth_enable': cfg.CONF.pecan.auth_enable,
+                'disable_cron_trigger_thread': True
             }
         })
-
-    @mock.patch.object(
-        auth_token.AuthProtocol, '_get_user_token_from_header',
-        mock.MagicMock(return_value=''))
-    @mock.patch.object(
-        auth_token.AuthProtocol, '_validate_token',
-        mock.MagicMock(return_value=PKI_TOKEN_VERIFIED))
-    @mock.patch.object(
-        db_api, "workbook_get",
-        mock.MagicMock(return_value=get_mock_workbook(WORKBOOKS[0])))
-    def test_auth_succeed(self):
-        resp = self.app.get('/v1/workbooks/my_workbook')
-        self.assertEqual(resp.status_int, 200)
-        self.assertDictEqual(WORKBOOKS[0], resp.json)
-
-    @mock.patch.object(
-        auth_token.AuthProtocol, '_get_user_token_from_header',
-        mock.MagicMock(return_value=''))
-    @mock.patch.object(
-        db_api, "workbook_get",
-        mock.MagicMock(return_value=get_mock_workbook(WORKBOOKS[0])))
-    def test_auth_fail(self):
-        # 401 unauthorized response is expected because the method
-        # _validate_user_token is not mocked in this test.
-        self.assertUnauthorized('/v1/workbooks/my_workbook')
