@@ -13,14 +13,18 @@
 #    limitations under the License.
 
 import datetime
+import eventlet
 import mock
 from oslo_config import cfg
 
+from mistral.engine.rpc_backend import rpc
 from mistral import exceptions as exc
+from mistral.services import periodic
 from mistral.services import security
 from mistral.services import triggers as t_s
 from mistral.services import workflows
-from mistral.tests import base
+from mistral.tests.unit import base
+from mistral import utils
 
 # Use the set_default method to set value otherwise in certain test cases
 # the change in value is not permanent.
@@ -39,6 +43,23 @@ my_wf:
       action: std.echo output='Hi!'
 """
 
+advance_cron_trigger_orig = periodic.advance_cron_trigger
+
+
+def new_advance_cron_trigger(ct):
+    """Wrap the original advance_cron_trigger method.
+
+    This method makes sure that the other coroutines will also run
+    while this thread is executing. Without explicitly passing control to
+    another coroutine the process_cron_triggers_v2 will finish looping
+    over all the cron triggers in one coroutine without any sharing at all.
+    """
+    eventlet.sleep()
+    modified = advance_cron_trigger_orig(ct)
+    eventlet.sleep()
+
+    return modified
+
 
 class TriggerServiceV2Test(base.DbTestCase):
     def setUp(self):
@@ -48,7 +69,7 @@ class TriggerServiceV2Test(base.DbTestCase):
 
     def test_trigger_create(self):
         trigger = t_s.create_cron_trigger(
-            'test',
+            'trigger-%s' % utils.generate_unicode_uuid(),
             self.wf.name,
             {},
             {},
@@ -70,9 +91,24 @@ class TriggerServiceV2Test(base.DbTestCase):
 
         self.assertEqual(datetime.datetime(2010, 8, 25, 0, 10), next_time)
 
+    def test_trigger_create_with_wf_id(self):
+        trigger = t_s.create_cron_trigger(
+            'trigger-%s' % utils.generate_unicode_uuid(),
+            None,
+            {},
+            {},
+            '*/5 * * * *',
+            None,
+            None,
+            datetime.datetime(2010, 8, 25),
+            workflow_id=self.wf.id
+        )
+
+        self.assertEqual(self.wf.name, trigger.workflow_name)
+
     def test_trigger_create_the_same_first_time_or_count(self):
         t_s.create_cron_trigger(
-            'test',
+            'trigger-%s' % utils.generate_unicode_uuid(),
             self.wf.name,
             {},
             {},
@@ -83,7 +119,7 @@ class TriggerServiceV2Test(base.DbTestCase):
         )
 
         t_s.create_cron_trigger(
-            'test2',
+            'trigger-%s' % utils.generate_unicode_uuid(),
             self.wf.name,
             {},
             {},
@@ -94,7 +130,7 @@ class TriggerServiceV2Test(base.DbTestCase):
         )
 
         t_s.create_cron_trigger(
-            'test3',
+            'trigger-%s' % utils.generate_unicode_uuid(),
             self.wf.name,
             {},
             {},
@@ -109,9 +145,9 @@ class TriggerServiceV2Test(base.DbTestCase):
         # But creation with the same count and first time
         # simultaneously leads to error.
         self.assertRaises(
-            exc.DBDuplicateEntryException,
+            exc.DBDuplicateEntryError,
             t_s.create_cron_trigger,
-            'test4',
+            'trigger-%s' % utils.generate_unicode_uuid(),
             self.wf.name,
             {},
             {},
@@ -136,7 +172,7 @@ class TriggerServiceV2Test(base.DbTestCase):
         exception = self.assertRaises(
             exc.InputException,
             t_s.create_cron_trigger,
-            'test',
+            'trigger-%s' % utils.generate_unicode_uuid(),
             'some_wf',
             {},
             {},
@@ -151,7 +187,7 @@ class TriggerServiceV2Test(base.DbTestCase):
 
     def test_oneshot_trigger_create(self):
         trigger = t_s.create_cron_trigger(
-            'test',
+            'trigger-%s' % utils.generate_unicode_uuid(),
             self.wf.name,
             {},
             {},
@@ -176,7 +212,7 @@ class TriggerServiceV2Test(base.DbTestCase):
         )
 
         trigger = t_s.create_cron_trigger(
-            'test',
+            'trigger-%s' % utils.generate_unicode_uuid(),
             self.wf.name,
             {},
             {},
@@ -189,8 +225,10 @@ class TriggerServiceV2Test(base.DbTestCase):
         self.assertEqual('my_trust_id', trigger.trust_id)
 
     def test_get_trigger_in_correct_orders(self):
+        t1_name = 'trigger-%s' % utils.generate_unicode_uuid()
+
         t_s.create_cron_trigger(
-            'test1',
+            t1_name,
             self.wf.name,
             {},
             {},
@@ -200,8 +238,10 @@ class TriggerServiceV2Test(base.DbTestCase):
             datetime.datetime(2010, 8, 25)
         )
 
+        t2_name = 'trigger-%s' % utils.generate_unicode_uuid()
+
         t_s.create_cron_trigger(
-            'test2',
+            t2_name,
             self.wf.name,
             {},
             {},
@@ -211,8 +251,10 @@ class TriggerServiceV2Test(base.DbTestCase):
             datetime.datetime(2010, 8, 22)
         )
 
+        t3_name = 'trigger-%s' % utils.generate_unicode_uuid()
+
         t_s.create_cron_trigger(
-            'test3',
+            t3_name,
             self.wf.name,
             {},
             {},
@@ -222,8 +264,10 @@ class TriggerServiceV2Test(base.DbTestCase):
             datetime.datetime(2010, 9, 21)
         )
 
+        t4_name = 'trigger-%s' % utils.generate_unicode_uuid()
+
         t_s.create_cron_trigger(
-            'test4',
+            t4_name,
             self.wf.name,
             {},
             {},
@@ -235,4 +279,59 @@ class TriggerServiceV2Test(base.DbTestCase):
 
         trigger_names = [t.name for t in t_s.get_next_cron_triggers()]
 
-        self.assertEqual(['test2', 'test1', 'test3'], trigger_names)
+        self.assertEqual([t2_name, t1_name, t3_name], trigger_names)
+
+    @mock.patch(
+        'mistral.services.periodic.advance_cron_trigger',
+        mock.MagicMock(side_effect=new_advance_cron_trigger)
+    )
+    @mock.patch.object(rpc.EngineClient, 'start_workflow')
+    def test_single_execution_with_multiple_processes(self, start_wf_mock):
+        def stop_thread_groups():
+            print('Killing cron trigger threads...')
+            [tg.stop() for tg in self.trigger_threads]
+
+        self.trigger_threads = [
+            periodic.setup(),
+            periodic.setup(),
+            periodic.setup()
+        ]
+        self.addCleanup(stop_thread_groups)
+
+        trigger_count = 5
+        t_s.create_cron_trigger(
+            'ct1',
+            self.wf.name,
+            {},
+            {},
+            '* * * * * */1',  # Every second
+            None,
+            trigger_count,
+            datetime.datetime(2010, 8, 25)
+        )
+
+        # Wait until there are 'trigger_count' executions.
+        self._await(
+            lambda: self._wait_for_single_execution_with_multiple_processes(
+                trigger_count,
+                start_wf_mock
+            )
+        )
+
+        # Wait some more and make sure there are no more than 'trigger_count'
+        # executions.
+        eventlet.sleep(5)
+
+        self.assertEqual(trigger_count, start_wf_mock.call_count)
+
+    def _wait_for_single_execution_with_multiple_processes(self, trigger_count,
+                                                           start_wf_mock):
+        eventlet.sleep(1)
+
+        return trigger_count == start_wf_mock.call_count
+
+    def test_get_next_execution_time(self):
+        pattern = '*/20 * * * *'
+        start_time = datetime.datetime(2016, 3, 22, 23, 40)
+        result = t_s.get_next_execution_time(pattern, start_time)
+        self.assertEqual(result, datetime.datetime(2016, 3, 23, 0, 0))
